package org.jdesktop.lg3d.wonderland.usecaseviewer.client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.lang.String;

import org.jdesktop.lg3d.wonderland.darkstar.client.ChannelController;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DListChangeMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.common.UCViewer2DTaskSelectMessage;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.ITask;
import org.jdesktop.lg3d.wonderland.usecaseviewer.controller.IProject;

import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DButtonBar;
import org.jdesktop.lg3d.wonderland.usecaseviewer.ui.UCViewer2DListViewer;

/**
 * UCViewer2DListDisplay.java
 * 
 * The panel for displaying the list of use cases for the current project to the user.
 * 
 * @author James Johnson
 * @author Matt Duval
 * @version 11/5/08
 *
 */
public class UCViewer2DListDisplay extends JPanel implements MouseListener {

	/**
	 * the application this panel is for
	 */
	private UCViewer2DApp app;

	/**
	 * the project list viewer
	 */
	protected UCViewer2DListViewer<ITask> listViewer;

	/**
	 * the button bar
	 */
	private UCViewer2DButtonBar buttonBar = new UCViewer2DButtonBar(5);
	private String[] texts = {"View", "Add", "Remove", "Projects", "Login"};
	
	/**
	 * sets up this panel for the provided app
	 * @param viewerApp
	 */
	public UCViewer2DListDisplay(UCViewer2DApp viewerApp){
		// set globals
		this.app = viewerApp;
		setBackground(Color.white);

		// load the taks from the current project, if any
		if(app.currentProject != null){
			listViewer = new UCViewer2DListViewer<ITask>("Tasks - " + app.currentProject.getProjectName(), (ArrayList<ITask>)app.currentProject.getAllTasks());
			listViewer.setupButtons(getWidth(), getHeight());
		}else{
			// create a blank to prevent ERROR
			listViewer = new UCViewer2DListViewer<ITask>("Tasks - no project selected", new ArrayList<ITask>());
			listViewer.setupButtons(getWidth(), getHeight());
		}
		
		// set up buttons
		try{
			buttonBar.setupButtons(getWidth(), getHeight(), texts);
		}catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
    /**
     * Set the size of the application
     * @param width the width of the application
     * @param height the height of the application
     */
    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
        if(listViewer != null) listViewer.setupButtons(width, height);
		try{
			buttonBar.setupButtons(width, height, texts);
		}catch(Exception e){
			e.printStackTrace();
		}
        repaint();
    }
	
	@Override
	public void paint(Graphics g){
		super.paint(g);
		
		// paint list
		if(listViewer != null) listViewer.paint(g, getWidth(), getHeight());

		// paint buttons
		buttonBar.paint(g);
	}
	
	public void mouseClicked(MouseEvent e){
		// get mouse point location
		Point p = e.getPoint();

		// find selection clicked
		int selectionChoice = -1;
		if(listViewer != null) selectionChoice = listViewer.selectionMade(p, getHeight(), getWidth());
		if(selectionChoice != -1){
			listViewer.selectedObject = selectionChoice;
			notifySelectedTaskChange(); // notify the server
			System.out.println("Selected task switched to " + listViewer.getSelectedObjectName());
			app.refreshUI();
			return;
		}

		// find if list viewer button clicked
		int listButtonChoice = -1;
		if(listViewer != null) listButtonChoice = listViewer.buttonClicked(p);
		switch(listButtonChoice){
		case 0:
			if(listViewer.selectedObject > 0) listViewer.selectedObject--;
			if(listViewer.selectedObject < listViewer.lowerBound) listViewer.lowerBound = listViewer.selectedObject;
			app.refreshUI();
			break;
		case 1:
			if(listViewer.selectedObject < listViewer.objectList.size() - 1) listViewer.selectedObject++;

			int maxDisplayCount = 0;
			int tempHeightValue = 7 * (getHeight() / 10);
			while(tempHeightValue > 45){
				tempHeightValue -= 45;
				maxDisplayCount++;
			}
			if(listViewer.selectedObject - listViewer.lowerBound >= maxDisplayCount) listViewer.lowerBound++;
			app.refreshUI();
			break;
		}
		
		// find button clicked
		int choice = buttonBar.buttonClicked(p);
		
		System.out.println("Button found " + choice);
		
		switch(choice){
		case -1: return; // by default
		case 0:
			// see if there is a selected task
			if(listViewer.selectedObject != -1){
				// check that their is a task
				if((listViewer.selectedObject >= 0)&&(listViewer.selectedObject < listViewer.objectList.size())){
					if(listViewer.objectList.get(listViewer.selectedObject) != null){
						// set current task
						notifySelectTask();
						setTaskToSelected();
							
						app.notifyViewDisplay();
						app.generateViewDisplay();
					}else{
						System.err.println("Error: null task at index " + listViewer.selectedObject);
					}
				}else{
					System.err.println("Warning: must select a task first");
				}
			}
			break;
		case 1:
			// add a new object
			if(app.currentProject != null){
				app.currentProject.addTask("New Task");
				listViewer.objectList = (ArrayList<ITask>)app.currentProject.getAllTasks();
				
				app.refreshUI();
			}
			break;
		case 2:
			// remove selected object
			// TODO
			break;
		case 3:
			app.notifyProjectSelectDisplay();
			app.generateProjectSelectDisplay();
			break;
		case 4:
			app.notifyLoginDisplay();
			app.generateLoginDisplay();
			break;
		}
	 }
	
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}
	public void mouseReleased(MouseEvent e) {}

	/**
	 * sets the task in the viewer to the selected task in the list
	 */
	protected void setTaskToSelected(){
		app.setCurrentTask((ITask) listViewer.objectList.get(listViewer.selectedObject));
	}
	
    /**
     * notifies the server that the selected task has changed to the selected index
     */
    public void notifySelectedTaskChange(){
    	UCViewer2DListChangeMessage msg = new UCViewer2DListChangeMessage(app.getCell().getCellID(), listViewer.selectedObject);
        ChannelController.getController().sendMessage(msg);
    }

    /**
     * notifies the server that to set the task to the selected task index
     */
    public void notifySelectTask(){
    	UCViewer2DTaskSelectMessage msg = new UCViewer2DTaskSelectMessage(app.getCell().getCellID(), listViewer.selectedObject);
        ChannelController.getController().sendMessage(msg);
    }
}
