package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Random;

/**
 * The XPlanner Database implementation of IProject
 * @author Ryan Moak
 */

public class DBProject implements IProject{
	String myName;
	int myId;
	ArrayList<ITask> tasks;
	
	
	/**
	 * Constructor for a DBProject
	 * @param name - the name of the DBProject
	 * @param id - the ID for the DBProject, for use by XPLanner
	 */
	public DBProject(String name, int id){
		myName = name;
		myId = id;
		tasks = new ArrayList<ITask>();
		//populates the task with the information from this ID
		DatabaseCommunicator.populateTasks(id, tasks);
	}

	/**
	 * Gets all the tasks for this project
	 * @return a Collection of ITasks
	 */
	public Collection<ITask> getAllTasks() {
			return tasks;
	}


	/**
	 * Gets the name of this Project
	 * @return the string representing the name 
	 */
	public String getProjectName() {
		return myName;
	}

	/**
	 * Gets the ID for this project
	 * @return returns the ID of this project
	 */
	public int getProjectID(){
		return myId;
	}

	/**
	 * Returns a task with the specified name
	 * @param the name of the task to find
	 * @return ITask with the specified name
	 */
	public ITask getTask(String name) {
		if (tasks == null){
			DatabaseCommunicator.populateTasks(getProjectID(), tasks);
		}
		ArrayList<ITask> tempTasks = (ArrayList<ITask>)getAllTasks();
		for (int i = 0; i < tempTasks.size(); i++){
			if (tempTasks.get(i).getName().equalsIgnoreCase(name)){
				return tempTasks.get(i);
			}		
		}
		return null;
	}


	/**
	 * Modifies an ITask, with the new Value
	 * @param newValue - the new Value to change the property to
	 * @param propertyToModify - the Property to modify
	 * @param task - the ITask to modify
	 */
	public void modifyTask(ITask task, String propertyToModify, String newValue) throws Exception {
		if (!checkProperty(propertyToModify)){
			throw new Exception ("Invalid Property");
		}
		task.setProperty(propertyToModify, newValue);
	}

	
	/**
	 * Checks the property name provided to see if it is valid
	 * @param property - the property to check
	 * @return boolean, true if the property is a valid one, false otherwise
	 */
	private boolean checkProperty(String property) {
		if (property.equals("name") || property.equals("description") || property.equals("type")
				|| property.equals("estimated_hours")){
			return true;
		}			
		return false;
	}

	/**
	 * Removes a specific Task from the database
	 * @param name - the name of the task to remove
	 * @return true, if the task was removed, false otherwise
	 */
	public boolean removeTask(String name) {
		DBTask taskToRemove = (DBTask) getTask(name);
		if (taskToRemove == null){
			return false;
		}
		boolean val = DatabaseCommunicator.removeTask(taskToRemove.getId());
		if (val){
			tasks = new ArrayList<ITask>();
			DatabaseCommunicator.populateTasks(getProjectID(), tasks);
			return true;
		}
		else {
			return false;
		}
	
	}
	
	
	/**
	 * Adds a task to this project
	 * @param the name of the project to add
	 * @return boolean, true if the task was added, false otherwise
	 */
	public boolean addTask(String name){
		int taskID = getNewID();
		
		int iterID = DatabaseCommunicator.getIterationID(getProjectID());
		if (iterID == -1){
			iterID = getNewID();
			DatabaseCommunicator.createNewIteration(getProjectID(), iterID);
		}
		
		int storyID = DatabaseCommunicator.getStoryID(iterID);
		if (storyID == -1){
			storyID = getNewID();
			DatabaseCommunicator.createNewStory(iterID, storyID);
		}
		
	
		boolean ret = DatabaseCommunicator.createNewTask(storyID, taskID, name);
		if (ret){
			DBTask newTask = new DBTask(name, taskID);
			tasks.add(newTask);
			return true;
		}
		else{
			return false;
		}
	}
	
	
	/**
	 * Returns a new random ID 
	 * @return int representing the ID
	 */
	private int getNewID(){
		Random r = new Random();
		int newID = r.nextInt(10000);
		System.out.println(newID);
		while(!DatabaseCommunicator.checkID(newID)){
			newID = r.nextInt(10000);
			System.out.println(newID);
		}
		return newID;
	}
}

