package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;
import java.net.URLConnection;

/**
 * Class used by the Pivotal Tracker controller in order to send requests to 
 * Pivotal Tracker
 * @author Ryan Moak
 */
public class HTTPCommunicator {

	
	private String username;
	private String token;
	
	/**
	 * Constructor for this object
	 */
	public HTTPCommunicator(){
		username = null;
		token = null;
	}
	
	
	/**
	 * Returns the username associated with this communicator
	 */
	public String getUsername(){
		return username;
	}
	
	/**
	 * Returns the token associated with this communicator, for use with Pivotal Tracker
	 */
	public String getToken(){
		return token; 
	}
	
	
	/**
	 * Sets the username for this communicator
	 * @param - the username to set
	 */
	public void setUsername(String user){
		username = user;
	}
	
	/**
	 * sets the token for this communicator
	 * @param - the token to set
	 */
	public void setToken(String tok){
		token = tok;
	}
	
	
	/**
	* Sends an HTTP GET request to a url
	*
	* @param endpoint - The URL of the server. (Example: " http://www.yahoo.com/search")
	* @param requestParameters - all the request parameters (Example: "param1=val1&param2=val2"). Note: This method will add the question mark (?) to the request - DO NOT add it yourself
	* @return - The response from the end point
	*/
	public String sendGetRequest(String endpoint, String requestParameters, boolean auth)
	{
		String result = null;
		if (endpoint.startsWith("http://"))
		{
			// Send a GET request to the servlet
			try
			{
				// Construct data
				StringBuffer data = new StringBuffer();

				// Send data
				String urlStr = endpoint;
				if (requestParameters != null && requestParameters.length () > 0)
				{
					urlStr += "?" + requestParameters;
				}
				URL url = new URL(urlStr);
				HttpURLConnection conn = (HttpURLConnection) url.openConnection ();
				conn.setRequestProperty("Content-type", "application/xml");
				
				if (auth){
					if (!getToken().equals(null)){
						conn.setRequestProperty("X-TrackerToken", getToken());
					}
					else {
						throw new Exception("A token must be set in order to use authenticated requests");
					}
				}
				
				conn.setRequestMethod("GET");
				// Get the response
				BufferedReader rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
				StringBuffer sb = new StringBuffer();
				String line;
				while ((line = rd.readLine()) != null)
				{
					sb.append(line + "\n");
				}
				rd.close();
				result = sb.toString();
			} catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		return result;
	}

	/**
	* Reads data from the data reader and send it to the endpoint.
	* @param - endpoint - The server's address
	* @param - request - the request to send
	* @param - type - the type of request to use, "PUT", "POST", or "DELETE"
	* @throws Exception
	*/
	public String postData(String endpoint, String request, String type) throws Exception
	{
		String result;
		URL postURL = new URL(endpoint);
		HttpURLConnection urlc = null;
		try
		{
			urlc = (HttpURLConnection) postURL.openConnection();
			try
			{
				urlc.setRequestMethod(type);
			} catch (ProtocolException e)
			{
				throw new Exception("Shouldn't happen: HttpURLConnection doesn't support POST??", e);
			}
			urlc.setDoOutput(true);
			urlc.setDoInput(true);
			urlc.setUseCaches(false);
			urlc.setAllowUserInteraction(false);
			urlc.setRequestProperty("Content-type", "application/xml; charset=UTF-8");
			
			if (getToken().equals(null)){
				throw new Exception("Token must be set in order to post requests");
			}
			
			urlc.setRequestProperty("X-TrackerToken", getToken());
		
			if (request!= null){
				OutputStream o = urlc.getOutputStream();
				try
				{
					Writer writer = new OutputStreamWriter(o, "UTF-8");
					writer.write(request);
					writer.close();
				} catch (IOException e)
				{
					throw new Exception("IOException while posting data", e);
				} finally
				{
					if (o != null)
						o.close();
				}
			}
			
			InputStream in = urlc.getInputStream();
			try
			{
				BufferedReader reader = new BufferedReader(new InputStreamReader(in));
				StringBuffer sb = new StringBuffer();
				String line;
				while ((line = reader.readLine()) != null)
				{
					sb.append(line + "\n");
				}
				result = sb.toString();
				reader.close();
			} catch (IOException e)
			{
				throw new Exception("IOException while reading response", e);
			} finally
			{
				if (in != null)
					in.close();
			}

		} catch (IOException e)
		{
			throw new Exception("Connection error (is server running at " + endpoint + " ?): " + e);
		} finally
		{
			if (urlc != null)
				urlc.disconnect();
		}
		return result;
	}


}

