package org.jdesktop.lg3d.wonderland.usecaseviewer.controller;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * PTProject.java
 * 
 * The project implementation for the Pivotal Tracker database.
 * 
 * @author Ryan Moak
 * @author James Johnson
 * @author Matt Duval
 *
 */
public class PTProject implements IProject{

	// the name of the project, and the id
	private String name;
	private int id;

	// the communicator
	private HTTPCommunicator comm;

	// the project tasks
	private HashMap<String, ITask> tasks;

	// the factories for the xml
	DOMParser parser = new DOMParser();
	XPathFactory xfactory = XPathFactory.newInstance();
	XPath path = xfactory.newXPath();

	/**
	 * sets up the new ptproject
	 * @param n - the name of this Project
	 * @param i - the id of this project
	 * @param communicator - the HTTPCommunicator to use to access PivotalTracker
	 */
	public PTProject(String n, int i, HTTPCommunicator communicator){
		name = n;
		id = i;
		tasks = null;
		comm = communicator;
	}

	/**
	 * adds a task with the provided name
	 */
	public boolean addTask(String name) {

		String addTaskUrl = "http://www.pivotaltracker.com/services/v1/projects/" + getID() + "/stories";

		String dataToPost = "<story><name>" + name + "</name><requested_by>" + comm.getUsername() + "</requested_by></story>";
		System.out.println(dataToPost);
		
		try {
			String result = comm.postData(addTaskUrl, dataToPost, "POST");

			// set up parser and document
			parser.parse(new InputSource(new StringReader(result)));
			Document taskDoc = parser.getDocument();
			
			int tempTaskID = Integer.parseInt(path.evaluate("/response/story/id", taskDoc));

			PTTask newtask = new PTTask(name, tempTaskID, getID(), null, comm);
			tasks.put(name, newtask);
		} catch (Exception e) {
			e.printStackTrace();
		}


		return false;
	}

	/**
	 * retrieves a list of all the tasks in the project
	 */
	public Collection<ITask> getAllTasks() {
		if (tasks == null) {
			try{
				tasks = new HashMap<String, ITask>();
				String getTasksUrl = "http://www.pivotaltracker.com/services/v1/projects/" + getID() + "/stories";

				// request and acquire the tasks
				String taskInfo = comm.sendGetRequest(getTasksUrl, null, true);

				// set up parser and document
				parser.parse(new InputSource(new StringReader(taskInfo)));
				Document taskDoc = parser.getDocument();

				if (taskInfo != null){

					// parse taskInfo 
					int taskCount = Integer.parseInt(path.evaluate("count(/response/stories/story)", taskDoc)); // get the task count
					// create tasks to add to the hashmap
					for(int t = 1; t <= taskCount; t++){
						// get name & id
						String tempTaskName = path.evaluate("/response/stories/story[" + t + "]/name", taskDoc);
						int tempTaskID = Integer.parseInt(path.evaluate("/response/stories/story[" + t + "]/id", taskDoc));

						// get properties
						Object[] props = new Object[6];
						props[0] = path.evaluate("/response/stories/story[" + t + "]/story_type", taskDoc); // story type
						props[1] = path.evaluate("/response/stories/story[" + t + "]/estimate", taskDoc); // estimate
						props[2] = path.evaluate("/response/stories/story[" + t + "]/description", taskDoc); // description
						props[3] = path.evaluate("/response/stories/story[" + t + "]/requested_by", taskDoc); // requested_by (creator)
						props[4] = path.evaluate("/response/stories/story[" + t + "]/current_state", taskDoc); // current_state (status)
						props[5] = path.evaluate("/response/stories/story[" + t + "]/owned_by", taskDoc); // owned_by (assigned)

						// create and add task to hash map
						PTTask tempTask = new PTTask(tempTaskName, tempTaskID, getID(), props, comm);
						tasks.put(tempTaskName, tempTask);
					}
				}
			}catch(SAXException e){
				e.printStackTrace();
			}catch(XPathExpressionException e){
				e.printStackTrace();
			}catch(IOException e){
				e.printStackTrace();
			}
		}
		return new ArrayList<ITask>(tasks.values());
	}

	/**
	 *@return the name of the project
	 */
	public String getProjectName() {
		return name;
	}

	public String toString(){
		return getProjectName();
	}

	/**
	 * @return the id of the project
	 */
	private int getID(){
		return id;
	}

	/**
	 * @return the task with the provided name
	 */
	public ITask getTask(String name) {
		if (tasks == null){
			getAllTasks();
		}
		if (tasks.containsKey(name)){
			return tasks.get(name);
		}
		return null;
	}

	/**
	 * modify the provided task property to be the new provided value
	 */
	public void modifyTask(ITask task, String propertyToModify, String newValue)
	throws Exception {
		task.setProperty(propertyToModify, newValue);
	}

	/**
	 * remove the task of the given name
	 * @return true if successful, false if otherwise
	 */
	public boolean removeTask(String name) {
		if (tasks.containsKey(name)){
			PTTask aTask = (PTTask) tasks.get(name);
			String removeTaskURL = "http://www.pivotaltracker.com/services/v1/projects/" + getID() + "/stories/" + aTask.getID();

			try {
				String result = comm.postData(removeTaskURL, null, "DELETE");

			} catch (Exception e) {

				e.printStackTrace();
			}


			boolean success = true;

			if(success){
				tasks.remove(name);
			}
		}
		return false;
	}

}
